/****************************************
 *  COPYRIGHT (C) 2012
 *  Holger Graf
 ****************************************/
package siarchive.imports;

import java.sql.SQLException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import siarchive.DataManager;
import siarchive.MainFrame;
import siarchive.persistence.Alliance;
import siarchive.persistence.DatabaseFactory;
import siarchive.persistence.dao.AllianceDao;

/**
 * @author graf
 *
 */
public class AllianceTagger
{
    private DataManager dataManager;
    private Pattern alliancePattern = Pattern.compile( "<\\s*a href.*?allianceID=(\\d+).*?>(.*?)</\\s*a>" );
    
    public AllianceTagger( DataManager dataManager )
    {
        this.dataManager = dataManager;
    }
    
    public void scan( String input ) throws SQLException
    {
        long now = System.currentTimeMillis();
        input = dataManager.cleanupInput( input );
        Matcher m = alliancePattern.matcher( input );
        AllianceDao allianceDao = DatabaseFactory.getDao( AllianceDao.class );
        Long account = dataManager.getActiveAccountId();
        while( m.find() )
        {
            long allianceId = MainFrame.parseLongAttribute( m.group( 1 ) );
            Alliance alliance = allianceDao.find( account, allianceId );
            if( alliance != null )
            {
                String tag = m.group( 2 ).trim();
                if( tag.length() > 0 )
                {
                    alliance.setTag( tag );
                    alliance.setUpdateTime( now );
                    allianceDao.save( alliance );
                }
            }
        }
        dataManager.commit();
    }
}
