/****************************************
 *  COPYRIGHT (C) 2012
 *  Holger Graf
 ****************************************/
package siarchive.persistence.dao;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import siarchive.components.Priority;
import siarchive.persistence.BattleReport;


/**
 * @author graf
 *
 */
public class BattleReportDao extends ReportDao<BattleReport>
{
    private final String FINDBYATTACKER = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND LOWER(ATTACKER) LIKE ?" + ORDERBYCREATIONTIMEASC;
    public List<BattleReport> findByAttacker(Long account, String attackerNameExpr) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYATTACKER, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setString( 3, sqlSubstitute( attackerNameExpr ).toLowerCase() );
        ResultSet resultSet = find.executeQuery();
        List<BattleReport> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }

    private final String FINDBYATTACKERANDPRIORITY = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND LOWER(ATTACKER) LIKE ? AND PRIORITY = ?" + ORDERBYCREATIONTIMEASC;
    public List<BattleReport> findByAttackerAndPriority(Long account, String attackerNameExpr, Priority priority) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYATTACKERANDPRIORITY, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setString( 3, sqlSubstitute( attackerNameExpr ).toLowerCase() );
        find.setString( 4, priority.name() );
        ResultSet resultSet = find.executeQuery();
        List<BattleReport> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }

    private final String FINDBYDEFENDER = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND LOWER(DEFENDER) LIKE ?" + ORDERBYCREATIONTIMEASC;
    public List<BattleReport> findByDefender(Long account, String defenderNameExpr) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYDEFENDER, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setString( 3, sqlSubstitute( defenderNameExpr ).toLowerCase() );
        ResultSet resultSet = find.executeQuery();
        List<BattleReport> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }

    private final String FINDBYDEFENDERANDPRIORITY = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND LOWER(DEFENDER) LIKE ? AND PRIORITY = ?" + ORDERBYCREATIONTIMEASC;
    public List<BattleReport> findByDefenderAndPriority(Long account, String defenderNameExpr, Priority priority) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYDEFENDERANDPRIORITY, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setString( 3, sqlSubstitute( defenderNameExpr ).toLowerCase() );
        find.setString( 4, priority.name() );
        ResultSet resultSet = find.executeQuery();
        List<BattleReport> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }

    @Override
    protected List<BattleReport> createDTO( ResultSet resultSet ) throws SQLException
    {
        List<BattleReport> list = new ArrayList<BattleReport>(); 
        while(resultSet.next())
        {
            Long id = resultSet.getLong( "ID" );
            BattleReport report = readCache(id);
            if(report == null )
            {
                report = new BattleReport();
                fillBaseDTO( report, resultSet );
                report.setAttacker( resultSet.getString( "ATTACKER" ) );
                report.setDefender( resultSet.getString( "DEFENDER" ) );
            }
            list.add(report);
            updateCache( report );
        }
        return list;
    }

    @Override
    protected String getKind()
    {
        return "battle";
    }

    @Override
    protected PreparedStatement reportCreateStatement( BattleReport report ) throws SQLException
    {
        PreparedStatement create = getConnection().prepareStatement( "INSERT INTO " + getTable() + " (KIND, ACCOUNT, CREATETIME, POSITION, ISASTEROID, PRIORITY, REPORTDATA, NOTES, ATTACKER, DEFENDER) values ('" + getKind() + "', ?, ?, ?, ?, ?, ?, ?, ?, ?)", Statement.RETURN_GENERATED_KEYS );
        create.setString( 8, report.getAttacker() );
        create.setString( 9, report.getDefender() );
        return create;
    }

    @Override
    protected PreparedStatement reportUpdateStatement( BattleReport report ) throws SQLException
    {
        PreparedStatement update;
        if(report.isRetainNotes()) 
        {
        	assert false;
            update = getConnection().prepareStatement( "UPDATE " + getTable() + " SET ACCOUNT = ?, CREATETIME = ?, POSITION = ?, ISASTEROID = ?, PRIORITY = ?, REPORTDATA = ?, ATTACKER = ?, DEFENDER = ? WHERE ID = ? AND KIND =  '" + getKind() + "'" );
            update.setString( 7, report.getAttacker() );
            update.setString( 8, report.getDefender() );
            update.setLong( 9, report.getId() );
        }
        else
        {
            update = getConnection().prepareStatement( "UPDATE " + getTable() + " SET ACCOUNT = ?, CREATETIME = ?, POSITION = ?, ISASTEROID = ?, PRIORITY = ?, REPORTDATA = ?, ATTACKER = ?, DEFENDER = ?, NOTES = ? WHERE ID = ? AND KIND =  '" + getKind() + "'" );
            update.setString( 7, report.getAttacker() );
            update.setString( 8, report.getDefender() );
            update.setString( 9, getNote( report.getNotes() ) );
            update.setLong( 10, report.getId() );
        }
        return update;
    }

}
