/****************************************
 *  COPYRIGHT (C) 2010
 *  Holger Graf
 ****************************************/
package siarchive.search;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.w3c.dom.Document;

import siarchive.DataManager;
import siarchive.MainFrame;
import siarchive.Parser;
import siarchive.components.DateTimeField;
import siarchive.components.EnumWrapper;
import siarchive.components.ImportExportPane;
import siarchive.components.ModulePane;
import siarchive.components.NoteBox;
import siarchive.components.Position;
import siarchive.components.PositionField;
import siarchive.components.Priority;
import siarchive.components.PriorityComparator;
import siarchive.components.SearchOption;
import siarchive.persistence.Account;
import siarchive.persistence.Planet;
import siarchive.persistence.SpyReport;
import siarchive.reports.SpyReportView;
import siarchive.table.AlternateLineTableCellRenderer;
import siarchive.table.NotesTableCellRenderer;
import siarchive.table.PointsRenderer;
import siarchive.table.PriorityListCellRenderer;
import siarchive.table.PriorityTableCellRenderer;
import siarchive.table.RowHeaderTableModel;
import siarchive.table.SpyReportTableModel;
import siarchive.table.TooltipTable;

/**
 * @author graf
 *
 */
public class SpyReportPane extends SearchPane implements ModulePane, ImportExportPane
{
    private static final long serialVersionUID = -2063793108262189275L;

    private MainFrame mainFrame;
    private JPanel center;
    private DataManager dataManager;

    private JTable table;
    private SpyReportTableModel model;
    private List<SpyReport> exportList = new ArrayList<SpyReport>();

    private JLabel searchLabel = new JLabel();
    private JComboBox box = new JComboBox();
    private JButton deleteSelected = new JButton();
    private JButton deleteAll = new JButton();
    private JButton exportSelected = new JButton();
    private JButton exportAll = new JButton();
    private JButton searchButton = new JButton();

    private JTextField planet = new JTextField();
    private JComboBox planetPriority = new JComboBox();
    private JTextField player = new JTextField();
    private JComboBox playerPriority = new JComboBox();
    private PositionField startPosition = new PositionField( false );
    private PositionField stopPosition = new PositionField( false );
    private JComboBox positionPriority = new JComboBox();
    private DateTimeField startTime = new DateTimeField();
    private DateTimeField stopTime = new DateTimeField();
    private JComboBox timePriority = new JComboBox();
    private CardLayout cardLayout;
    private JPanel cardPanel;
    private TableRowSorter<TableModel> sorter;
    private PriorityComparator priorityComparator = new PriorityComparator();

    private Account currentAccount = null;
    private JMenuItem popupGoto = new JMenuItem();
    private JMenuItem popupShow = new JMenuItem();
    private Point menuPosition = null;

    /**
     * @param parent
     */
    public SpyReportPane( MainFrame parent )
    {
        super();
        this.mainFrame = parent;
        this.dataManager = parent.getDataManager();
        setLayout( new BorderLayout() );
        setBorder( null );
        center = createCenterPanel();
        add( center, BorderLayout.CENTER );
    }

    protected JPanel createCenterPanel()
    {
        JPanel centerPanel = new JPanel( new BorderLayout() );
        JPanel controlPanel = new JPanel( new BorderLayout() );
        controlPanel.setBorder( BorderFactory.createEmptyBorder( 2, 3, 15, 3) );
        JPanel boxPanel = new JPanel( new GridLayout( 2, 1, 0, 0 ) );
        updateSearchOptions();
        boxPanel.add( searchLabel );
        boxPanel.add( box );
        controlPanel.add( boxPanel, BorderLayout.NORTH );

        cardLayout = new CardLayout();
        JPanel optionPanel = new JPanel( new BorderLayout() );
        cardPanel = new JPanel( cardLayout );
        JPanel panel;
        panel = new JPanel( new GridLayout( 3, 1) );
        panel.add( planet );
        panel.add( planetPriority );
        panel.add( new JLabel() );
        cardPanel.add( panel, SearchOption.planets.name() );
        panel = new JPanel( new GridLayout( 3, 1) );
        panel.add( player );
        panel.add( playerPriority );
        panel.add( new JLabel() );
        cardPanel.add( panel, SearchOption.players.name() );
        panel = new JPanel(new GridLayout( 3, 1 ));
        panel.add( startPosition );
        panel.add( stopPosition );
        panel.add( positionPriority );
        cardPanel.add( panel, SearchOption.systems.name() );
        panel = new JPanel(new GridLayout( 3, 1 ));
        panel.add( startTime );
        panel.add( stopTime );
        panel.add( timePriority );
        cardPanel.add( panel, SearchOption.time.name() );
        optionPanel.add( cardPanel, BorderLayout.NORTH );
        JPanel buttonPanel = new JPanel( new BorderLayout() );
        buttonPanel.add( searchButton, BorderLayout.NORTH );
        optionPanel.add( buttonPanel, BorderLayout.CENTER );
        controlPanel.add( optionPanel, BorderLayout.CENTER );

        box.addItemListener( new ItemListener()
        {
            @SuppressWarnings("unchecked")
            public void itemStateChanged( ItemEvent e )
            {
                if( e.getStateChange() == ItemEvent.SELECTED )
                {
                    SearchOption searchOption = ((EnumWrapper<SearchOption>)e.getItem()).getBase();
                    cardLayout.show( cardPanel, searchOption.name() );
                    switch ( searchOption )
                    {
                        case planets:
                        case systems:
                            sorter.toggleSortOrder( SpyReportTableModel.PLANET );
                            break;
                        case time:
                            sorter.toggleSortOrder( SpyReportTableModel.TIME );
                            break;
                        default:
                            break;
                    }
                }
            }

        });
        searchButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionSearch();
            }
        } );

        buttonPanel = new JPanel( new GridLayout( 4, 1, 0, 0 ) );
        buttonPanel.add( exportSelected );
        buttonPanel.add( exportAll );
        buttonPanel.add( deleteSelected );
        buttonPanel.add( deleteAll );
        controlPanel.add( buttonPanel, BorderLayout.SOUTH );
        centerPanel.add( controlPanel, BorderLayout.WEST );

        final RowHeaderTableModel rowModel = new RowHeaderTableModel();
        final int width = 46;

        JTable rowHeader = new JTable( rowModel )
        {
            private static final long serialVersionUID = -8707317721680383319L;

            public Dimension getPreferredScrollableViewportSize()
            {
                Dimension size = getPreferredSize();
                size.width = width;
                return size;
            }
        };
        rowHeader.setAutoResizeMode( JTable.AUTO_RESIZE_OFF );
        TableColumnModel columnModel = rowHeader.getColumnModel();
        TableColumn column;
        column = columnModel.getColumn( 0 );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );

        model = new SpyReportTableModel( dataManager );
        model.addTableModelListener( new TableModelListener()
        {
            public void tableChanged( TableModelEvent e )
            {
                TableModel tableModel = ( TableModel )e.getSource();
                rowModel.setRows( tableModel.getRowCount() );
            }
        } );
        final TableCellRenderer priorityRenderer = new PriorityTableCellRenderer();
        final TableCellRenderer notesRenderer = new NotesTableCellRenderer();
        JComboBox editor = new JComboBox();
        editor.addItem( "---" );
        for( Priority priority : Priority.values() )
        {
            editor.addItem( priority );
        }
        editor.setRenderer( new PriorityListCellRenderer() );
        final TableCellEditor priorityEditor = new DefaultCellEditor( editor );
        table = new TooltipTable( model )
        {
            private static final long serialVersionUID = -8499738896262825180L;

            @Override
            public TableCellEditor getCellEditor( int row, int column )
            {
                TableCellEditor rv = super.getCellEditor();
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );

                switch( modelColumn )
                {
                    case SpyReportTableModel.PRIORITY:
                        rv = priorityEditor;
                        break;
                }
                return rv;
            }

            @Override
            public TableCellRenderer getCellRenderer( int row, int column )
            {
                TableCellRenderer rv = super.getCellRenderer( row, column );
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );

                switch( modelColumn )
                {
                    case SpyReportTableModel.PRIORITY:
                        rv = priorityRenderer;
                        break;
                    case SpyReportTableModel.NOTES:
                        rv = notesRenderer;
                        break;
                }
                return rv;
            }

        };
        JScrollPane scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        table.setDefaultRenderer( String.class, new AlternateLineTableCellRenderer() );
        table.setDefaultRenderer( Number.class, new PointsRenderer() );
        sorter = new TableRowSorter<TableModel>();
        sorter.setModel( model );
        sorter.setComparator( SpyReportTableModel.PRIORITY, priorityComparator );
        sorter.setMaxSortKeys( 3 );
        table.setRowSorter( sorter );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        scrollPane.setRowHeaderView( rowHeader );
        scrollPane.setCorner( ScrollPaneConstants.UPPER_LEFT_CORNER, rowHeader.getTableHeader() );
        PointsRenderer rowRenderer = new PointsRenderer();
        rowRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
        rowRenderer.setBorder( null );
        rowHeader.setDefaultRenderer( Integer.class, rowRenderer );
        centerPanel.add( scrollPane, BorderLayout.CENTER );

        table.getSelectionModel().addListSelectionListener( new ListSelectionListener()
        {
            public void valueChanged( ListSelectionEvent e )
            {
                if( !e.getValueIsAdjusting() )
                {
                    boolean enable = ( e.getFirstIndex() >= 0 );
                    deleteSelected.setEnabled( enable );
                    exportSelected.setEnabled( enable );
                }
            }
        });
        deleteAll.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                mainFrame.dbOn();
                try
                {
                    model.removeAll();
                }
                catch( SQLException ex )
                {
                    mainFrame.showErrorInfo( ex );
                }
                mainFrame.dbOff();
                mainFrame.updatePane();
            }

        });
        deleteSelected.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                int[] rows = table.getSelectedRows();
                int[] modelRows = new int[rows.length];
                for( int i = 0; i < rows.length; i++ )
                {
                    modelRows[i] = table.convertRowIndexToModel( rows[i] );
                }
                mainFrame.dbOn();
                try
                {
                    model.removeRows( modelRows );
                }
                catch( SQLException ex )
                {
                    mainFrame.showErrorInfo( ex );
                }
                mainFrame.dbOff();
                mainFrame.updatePane();
            }

        });
        exportSelected.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                int[] rows = table.getSelectedRows();
                int[] modelRows = new int[rows.length];
                for( int i = 0; i < rows.length; i++ )
                {
                    modelRows[i] = table.convertRowIndexToModel( rows[i] );
                }
                exportList.clear();
                exportList.addAll( model.getExportRows( modelRows ) );
                dataManager.saveData( SpyReportPane.this );
            }

        });
        exportAll.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                exportList.clear();
                exportList.addAll( model.getExportRows() );
                dataManager.saveData( SpyReportPane.this );
            }

        });
        popupGoto.addActionListener( new ActionListener()
        {

            public void actionPerformed( ActionEvent e )
            {
                actionGoto();
            }
        } );
        popupShow.addActionListener( new ActionListener()
        {

            public void actionPerformed( ActionEvent e )
            {
                actionShow();
            }
        } );
        table.addMouseListener( new MouseAdapter()
        {
            @Override
            public void mousePressed( MouseEvent event )
            {
                if( event.isPopupTrigger() )
                {
                    JPopupMenu popup = new JPopupMenu();
                    popup.add( popupShow );
                    popup.add( popupGoto );
                    popup.pack();

                    menuPosition = new Point( event.getX(), event.getY() );
                    popup.show(table, event.getX(), event.getY() );
                }
            }

            @Override
            public void mouseReleased( MouseEvent event )
            {
                if( event.isPopupTrigger() )
                {
                    JPopupMenu popup = new JPopupMenu();
                    popup.add( popupShow );
                    popup.add( popupGoto );
                    popup.pack();

                    menuPosition = new Point( event.getX(), event.getY() );
                    popup.show(table, event.getX(), event.getY() );
                }
            }

            @Override
            public void mouseClicked( MouseEvent event )
            {
                if( event.getClickCount() >= 2 )
                {
                    Point point = new Point( event.getX(), event.getY() );
                    int modelColumn = table.convertColumnIndexToModel( table.columnAtPoint( point ) );
                    int modelRow = table.convertRowIndexToModel( table.rowAtPoint( point ) );
                    switch( modelColumn )
                    {
                        case SpyReportTableModel.NOTES:
                            SpyReport report = model.getRow( modelRow );
                            NoteBox<SpyReport> box = new NoteBox<SpyReport>( mainFrame, report );
                            mainFrame.showModalBox( box );
                            model.fireTableCellUpdated( modelRow, modelColumn );
                            break;
                        case SpyReportTableModel.PRIORITY:
                            break;
                        default:
                            menuPosition = new Point( event.getX(), event.getY() );
                            actionShow();
                            break;
                    }
                }
            }

        } );

        planetPriority.setRenderer( new PriorityListCellRenderer() );
        playerPriority.setRenderer( new PriorityListCellRenderer() );
        timePriority.setRenderer( new PriorityListCellRenderer() );
        positionPriority.setRenderer( new PriorityListCellRenderer() );

        return centerPanel;
    }

    protected void actionGoto()
    {
        int viewRow = table.rowAtPoint( menuPosition );
        int modelRow = table.convertRowIndexToModel( viewRow );
        if( modelRow >= 0  )
        {
            SpyReport report = model.getRow( modelRow );
            Position position = Position.createFromId( report.getPosition() );
            mainFrame.showGalaxyView( position.getGalaxy(), position.getSystem(), true );
        }

    }

    protected void actionShow()
    {
        int viewRow = table.rowAtPoint( menuPosition );
        int modelRow = table.convertRowIndexToModel( viewRow );
        if( modelRow >= 0  )
        {
            SpyReport report = model.getRow( modelRow );
            SpyReportView spyReportViewer = new SpyReportView( report, mainFrame );
            spyReportViewer.setVisible( true );
        }

    }

    @SuppressWarnings("unchecked")
    protected void actionSearch()
    {
        SearchOption searchOption = ((EnumWrapper<SearchOption>)box.getSelectedItem()).getBase();
        Priority priority = null;
        Collection<SpyReport> result = null;
        mainFrame.dbOn();
        try
        {
            switch( searchOption )
            {
                case planets:
                    if( planetPriority.getSelectedItem() instanceof Priority )
                    {
                        priority = ( Priority )planetPriority.getSelectedItem();
                    }
                    result = dataManager.findSpyReportsByPlanetAndPriority( planet.getText(), priority );
                    break;
                case players:
                    if( playerPriority.getSelectedItem() instanceof Priority )
                    {
                        priority = ( Priority )playerPriority.getSelectedItem();
                    }
                    result = dataManager.findSpyReportsByPlayerAndPriority( player.getText(), priority );
                    break;
                case time:
                    if( timePriority.getSelectedItem() instanceof Priority )
                    {
                        priority = ( Priority )timePriority.getSelectedItem();
                    }
                    result = dataManager.findSpyReportsByTimeAndPriority( startTime.getDate().getTime(), stopTime.getDate().getTime(), priority );
                    break;
                case systems:
                    if( positionPriority.getSelectedItem() instanceof Priority )
                    {
                        priority = ( Priority )positionPriority.getSelectedItem();
                    }
                    result = dataManager.findSpyReportsBySystemsAndPriority( startPosition.getPosition( 1 ), stopPosition.getPosition( 16 ), priority );
                    break;
            }
        }
        catch( Exception ex )
        {
            mainFrame.showErrorInfo( ex );
        }
        mainFrame.dbOff();
        if( result != null )
        {
            model.setData( result );
        }
    }

    private boolean accountChanged()
    {
        Account newAccount = dataManager.getActiveAccount();
        return( currentAccount != null && !currentAccount.equals( newAccount )
         || newAccount != null && !newAccount.equals( currentAccount )
        );
    }

    public void update()
    {
        mainFrame.getRootPane().setDefaultButton( searchButton );
        updateLabel();
        updateTable();
        currentAccount = dataManager.getActiveAccount();
    }

    private void updateTable()
    {
        // clear table on account change only
        if( accountChanged() )
        {
            model.update();
            sorter.setComparator( SpyReportTableModel.PRIORITY, priorityComparator );
            sorter.setMaxSortKeys( 3 );
        }
        table.setRowHeight( 16 );

        int width = 20;
        adjustColumnWidth( SpyReportTableModel.PRIORITY, width );
        adjustColumnWidth( SpyReportTableModel.NOTES, width );

        updateColumnNames();
    }

    @Override
    protected JTable getTable()
    {
        return table;
    }

    private void updateLabel()
    {
        if( accountChanged() )
        {
            updateSearchOptions();
        }
        searchLabel.setText( dataManager.getI18nText( "Search.search" ) );
        deleteSelected.setText( dataManager.getI18nText( "delete" ) );
        deleteAll.setText( dataManager.getI18nText( "deleteAll" ) );
        exportSelected.setText( dataManager.getI18nText( "export" ) );
        exportAll.setText( dataManager.getI18nText( "exportAll" ) );
        searchButton.setText( dataManager.getI18nText( "Search.searchCommand" ) );
        popupGoto.setText( dataManager.getI18nText( "Popup.goto" ) );
        popupShow.setText( dataManager.getI18nText( "Popup.show" ) );

        deleteSelected.setEnabled( false );
        deleteAll.setEnabled( true );
        exportSelected.setEnabled( false );
        exportAll.setEnabled( true );
    }

    @SuppressWarnings("unchecked")
    private void updateSearchOptions()
    {
        Object selectedItem = box.getSelectedItem();
        SearchOption option = SearchOption.planets;
        if( selectedItem != null )
        {
            option = ((EnumWrapper<SearchOption>)selectedItem).getBase();
        }
        box.removeAllItems();
        SearchOption[] options = SearchOption.forSpyReports();
        for( SearchOption searchOption : options )
        {
            box.addItem( new EnumWrapper<SearchOption>( dataManager, searchOption ) );
        }
        box.setSelectedItem( new EnumWrapper<SearchOption>( dataManager, option ) );
        box.setMaximumRowCount( options.length );

        planetPriority.removeAllItems();
        playerPriority.removeAllItems();
        positionPriority.removeAllItems();
        timePriority.removeAllItems();
        Priority[] priorities = Priority.values();
        planetPriority.addItem( "---" );
        playerPriority.addItem( "---" );
        positionPriority.addItem( "---" );
        timePriority.addItem( "---" );
        for( Priority priority : priorities )
        {
            planetPriority.addItem( priority );
            playerPriority.addItem( priority );
            positionPriority.addItem( priority );
            timePriority.addItem( priority );
        }
        planet.setText( "*" );
        player.setText( "*" );
        startPosition.setMaxGalaxies(Position.getUpperGalaxyBound(dataManager.getFlavor()));
        stopPosition.setMaxGalaxies(Position.getUpperGalaxyBound(dataManager.getFlavor()));
    }

    public File getCurrentDir()
    {
        return dataManager.getCurrentDir();
    }

    public Document getData( Parser xmlParser )
    {
        Document doc = null;
        if( xmlParser != null && dataManager.getActiveAccount() != null )
        {
            doc = xmlParser.getDocument();
            dataManager.createAppRoot( doc );
            mainFrame.exportSpyReports( doc, exportList );
        }
        return doc;
    }

    public String getExtension()
    {
        return dataManager.getExtension();
    }

    public String getFileName()
    {
        StringBuilder sb = new StringBuilder();
        sb.append( DataManager.getFileFormat().format( new Date() ) ).append( ' ' );
        sb.append( dataManager.getI18nText( "SpyReport.spyreport" ) );
        return sb.toString();
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    public void setCurrentDir( File currentDir )
    {
        dataManager.setCurrentDir( currentDir );
    }

    public void setPlanet( Planet planet )
    {
        update();
        Position position = Position.createFromId( planet.getPosition() );
        this.startPosition.setPosition( position );
        this.stopPosition.setPosition( position );
        this.positionPriority.setSelectedIndex( 0 );
        this.box.setSelectedItem( new EnumWrapper<SearchOption>( dataManager, SearchOption.systems ) );
        this.cardLayout.show( cardPanel, SearchOption.systems.name() );
        actionSearch();
    }
}
