package siarchive.table;

import java.awt.Color;
import java.awt.Component;

import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellRenderer;

public class CheckBoxTableCellRenderer implements TableCellRenderer
{
    private JCheckBox component = new JCheckBox();
    private TableCellRenderer delegate;
    private Icon selectedIcon = null;
    private Icon unselectedIcon = null;
    private JLabel blank = new JLabel();
    
    public CheckBoxTableCellRenderer()
    {
        super();
        this.delegate = new AlternateLineTableCellRenderer();
        component.setHorizontalAlignment( SwingConstants.CENTER );
        blank.setOpaque( true );
    }

    public CheckBoxTableCellRenderer( Icon selectedIcon, Icon unselectedIcon )
    {
        this();
        this.selectedIcon = selectedIcon;
        this.unselectedIcon = unselectedIcon;
    }

    public Component getTableCellRendererComponent (JTable table, 
                    Object value, boolean isSelected, boolean hasFocus, int row, int column) 
    {
        Component c = delegate.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        Color bg = c.getBackground();
        c = component;
        // Configure the component with the specified value
    	component.setSelected( Boolean.parseBoolean( value.toString() ) );
    	if( selectedIcon != null && unselectedIcon != null )
    	{
    		if( value instanceof Boolean )
	    	{
	    		Icon icon = ( component.isSelected() ) ? selectedIcon : unselectedIcon;
	    		component.setIcon( icon );
	    	}
    		else
	        {
    			c = blank;
	        }
    	}        
    	c.setBackground( bg );
        return c;
    }

    public void setHorizontalAlignment( int alignment )
    {
        component.setHorizontalAlignment( alignment );
    }
}
