/****************************************
 *  COPYRIGHT (C) 2012
 *  Holger Graf
 ****************************************/
package siarchive.table;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

/**
 * @author graf
 *
 */
public class HideTableColumnModel extends DefaultTableColumnModel
{
    private static final long serialVersionUID = 5023083897619542542L;
    private List<TableColumn> allColumns = new ArrayList<TableColumn>();

    @Override
    public void addColumn( TableColumn aColumn )
    {
        allColumns.add( aColumn );
        super.addColumn( aColumn );
    }

    @Override
    public void removeColumn( TableColumn column )
    {
        allColumns.remove( column );
        super.removeColumn( column );
    }

    @Override
    public void moveColumn( int columnIndex, int newIndex )
    {
        // lifted from DefaultTableModel, must check first
        if( ( columnIndex < 0 ) || ( columnIndex >= getColumnCount() ) ||
            ( newIndex < 0 ) || ( newIndex >= getColumnCount() ) )
        {
            throw new IllegalArgumentException("moveColumn() - Index out of range");
        }
        // same as above
        if( columnIndex == newIndex )
        {
            super.moveColumn( columnIndex, newIndex );
            return;
        }
        
        TableColumn column = tableColumns.get( columnIndex );
        TableColumn newColumn = tableColumns.get( newIndex );
        
        // move in all
        int allNewIndex = allColumns.indexOf( newColumn );
        allColumns.remove( column );
        allColumns.add( allNewIndex, column );

        // finally move in base
        super.moveColumn( columnIndex, newIndex );
    }

    public boolean isColumnVisible( int modelIndex )
    {
        boolean rv = false;
        // valid column?
        TableColumn column = findByModelIndex( modelIndex, allColumns );
        if( column != null )
        {
            rv = tableColumns.contains( column );
        }
        return rv;
    }
    
    public void setColumnVisible( int modelIndex, boolean isVisible )
    {
        // valid column?
        TableColumn column = findByModelIndex( modelIndex, allColumns );
        if( column != null )
        {
            if( isVisible )
            {
                if( !tableColumns.contains( column ) )
                {
                    int toIndex = nextVisibleIndex( column );
                    super.addColumn( column );
                    super.moveColumn( tableColumns.size() - 1, toIndex );
                }
            }
            else
            {
                // just remove from base
                super.removeColumn( column );
            }
        }
    }
    
    private int nextVisibleIndex( TableColumn column )
    {
        int rv = tableColumns.size();
        int index = allColumns.indexOf( column ) + 1;
        // last element?
        if( index < allColumns.size() )
        {
            // get next visible column after
            column = allColumns.get( index );
            while( !tableColumns.contains( column ) && ++index < allColumns.size() )
            {
                column = allColumns.get( index );
            }
            if( index < allColumns.size() )
            {
                rv = tableColumns.indexOf( column );
            }
        }
        return rv;
    }

    private TableColumn findByModelIndex( int modelIndex, Collection<TableColumn> columns )
    {
        TableColumn column = null;
        for( TableColumn tc : columns )
        {
            if( tc.getModelIndex() == modelIndex )
            {
                column = tc;
                break;
            }
        }
        return column;
    }
}
