/****************************************
 *  COPYRIGHT (C) 2010
 *  Holger Graf
 ****************************************/
package siarchive.table;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import siarchive.DataManager;
import siarchive.components.Position;
import siarchive.components.Priority;
import siarchive.components.Status;
import siarchive.persistence.Note;
import siarchive.persistence.SpyReport;

/**
 * @author graf
 *
 */
public class SpyReportTableModel extends AbstractTableModel
{
    public static final int TIME = 0;
    public static final int PLANET = TIME + 1;
    public static final int NOTES = PLANET + 1 ;
    public static final int PRIORITY = NOTES + 1 ;
    
    private static final long serialVersionUID = -1770345678362128348L;
    private DataManager dataManager;
    private List<SpyReport> data = new ArrayList<SpyReport>();
    /**
     * 
     */
    public SpyReportTableModel( DataManager dataManager )
    {
        this.dataManager = dataManager;
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return PRIORITY + 1;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return data.size();
    }

    public String getColumnName( int columnIndex )
    {
        String rv ="";
        switch( columnIndex )
        {
            case PLANET:
                rv = dataManager.getI18nText( "PlanetView.coords" );
                break;
            case TIME:
                rv = dataManager.getI18nText( "Report.time" );
                break;
            case PRIORITY:
                rv = dataManager.getI18nText( "Report.priority" );
                break;
            case NOTES:
                rv = "/";
                break;
        }
        return rv;
    }
    
    
    @Override
    public Class<?> getColumnClass( int columnIndex )
    {
        Class<?> rv = String.class;
        switch( columnIndex )
        {
            case PLANET:
            case TIME:
                rv = String.class;
                break;
            case PRIORITY:
                rv = Status.class;
                break;
            case NOTES:
                rv = Note.class;
                break;
        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        SpyReport report = data.get( rowIndex );
        switch( columnIndex )
        {
            case PLANET:
                StringBuilder sb = new StringBuilder();
                sb.append( Position.createFromId( report.getPosition() ).toString() );
                sb.append( ' ' );
                sb.append( report.getPlanetName() );
                rv = sb.toString();
                break;
            case PRIORITY:
                rv = report.getFlag();
                break;
            case NOTES:
                rv = report.getNotes();
                break;
            case TIME:
                rv = DataManager.getDateFormat().format( new Date( report.getCreationTime() ) );
        }
        return rv;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return ( columnIndex == PRIORITY );
    }
    
    public void setValueAt(Object value, int rowIndex, int columnIndex)
    {
        switch( columnIndex )
        {
            case PRIORITY:
                Priority priority = null;
                if( value instanceof Priority)
                {
                    priority = ( Priority )value;
                }
                SpyReport report = data.get( rowIndex );
                report.setFlag( priority );
                try
                {
                    dataManager.save( report );
                    dataManager.commit();
                }
                catch( SQLException e )
                {
                    e.printStackTrace();
                }
                break;
        }
    }
    
    public SpyReport getRow( int row )
    {
        return data.get( row );
    }
    
    public void removeRows( int[] modelRows ) throws SQLException
    {
        // delete back to front to preserve indices
        Arrays.sort( modelRows );
        List<SpyReport> oldReports = new ArrayList<SpyReport>();
        for( int i = modelRows.length - 1; i >= 0; i-- )
        {
            oldReports.add(  data.remove( modelRows[i] ) );
        }
        dataManager.removeReports( oldReports );
        dataManager.commit();
        fireTableDataChanged();
    }

    public List<SpyReport> getExportRows( int[] modelRows )
    {
        List<SpyReport> result = new ArrayList<SpyReport>();
        for( int i = 0; i < modelRows.length; i++ )
        {
            result.add( data.get( modelRows[i] ) );
        }
        return result;
    }

    public List<SpyReport> getExportRows()
    {
        List<SpyReport> result = new ArrayList<SpyReport>();
        result.addAll( data );
        return result;
    }

    public void removeAll() throws SQLException
    {
        dataManager.removeReports( data );
        dataManager.commit();
        data.clear();
        fireTableDataChanged();
    }

    public void setData( Collection<SpyReport> newData )
    {
        data.clear();
        data.addAll( newData );
        fireTableDataChanged();
    }
    
    public void update()
    {
        data.clear();
        // repaint header
        fireTableStructureChanged();
    }
}
