/****************************************
 *  COPYRIGHT (C) 2009
 *  Holger Graf
 ****************************************/
package siarchive.table;

import java.awt.Font;
import java.awt.event.MouseEvent;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import siarchive.DataManager;
import siarchive.components.Cost;
import siarchive.persistence.Alliance;
import siarchive.persistence.Note;
import siarchive.persistence.Planet;
import siarchive.persistence.Player;

/**
 * @author graf
 *
 */
public class TooltipTable extends JTable
{
	private static final long serialVersionUID = 5374293775888082770L;

	private Font font;
	
    public TooltipTable()
    {
        super();
    }

    /**
     * @param numRows
     * @param numColumns
     */
    public TooltipTable( int numRows, int numColumns )
    {
        super( numRows, numColumns );
    }

    /**
     * @param rowData
     * @param columnNames
     */
    public TooltipTable( Object[][] rowData, Object[] columnNames )
    {
        super( rowData, columnNames );
    }

    /**
     * @param dm
     * @param cm
     * @param sm
     */
    public TooltipTable( TableModel dm, TableColumnModel cm, ListSelectionModel sm )
    {
        super( dm, cm, sm );
    }

    /**
     * @param dm
     * @param cm
     */
    public TooltipTable( TableModel dm, TableColumnModel cm )
    {
        super( dm, cm );
    }

    /**
     * @param dm
     */
    public TooltipTable( TableModel dm )
    {
        super( dm );
    }

    @Override
    protected TableColumnModel createDefaultColumnModel()
    {
        return new HideTableColumnModel();
    }

    protected JTableHeader createDefaultTableHeader() 
    {
        return new TooltipTableHeader(columnModel);
    }

    @Override
	public String getToolTipText(MouseEvent event) 
    {
        String tooltip = null;
        int viewRow = rowAtPoint( event.getPoint() );
        int viewColumn = columnAtPoint( event.getPoint() );
        int modelRow = convertRowIndexToModel( viewRow );
        int modelColumn = convertColumnIndexToModel( viewColumn );
        if( modelRow >= 0 && modelColumn >= 0 )
        {
	        Object value = getModel().getValueAt( modelRow, modelColumn );
	        if( value instanceof String )
	        {
	        	tooltip = value.toString();
	        	if( tooltip.trim().length() == 0 )
	        	{
	        		tooltip = null;
	        	}
	        }
	        else if( value instanceof Number )
	        {
	            Number number = ( Number )value;
	        	if( number.longValue() != 0 )
	        	{
	        		tooltip = DataManager.format( number.longValue() );
	        	}
	        }
            else if( value instanceof Planet )
            {
                tooltip = value.toString();
                if( tooltip.trim().length() == 0 )
                {
                    tooltip = null;
                }
            }
            else if( value instanceof Alliance )
            {
                Alliance alliance = ( Alliance )value;
                StringBuilder sb = new StringBuilder();
                sb.append( alliance.toString() );
                if( alliance.getAllianceId() > 0 )
                {
                    sb.append( " [id:" ).append( alliance.getAllianceId() ).append( ']' );
                }
                tooltip = sb.toString();
                if( tooltip.trim().length() == 0 )
                {
                    tooltip = null;
                }
            }
            else if( value instanceof Player )
            {
                Player player = ( Player )value;
                StringBuilder sb = new StringBuilder();
                sb.append( player.getName() );
                if( player.getPlayerId() >= 0 )
                {
                    sb.append( " [uid:" ).append( player.getPlayerId() ).append( ']' );
                }
                tooltip = sb.toString();
                if( tooltip.trim().length() == 0 )
                {
                    tooltip = null;
                }
            }
            else if( value instanceof Note )
            {
                tooltip = value.toString();
                if( tooltip.trim().length() == 0 )
                {
                    tooltip = null;
                }
            }
            else if( value instanceof Cost && getModel() instanceof CostRenderer )
            {
                tooltip = ((CostRenderer)getModel()).renderCost( ( Cost )value );
            }
        }
        return tooltip;
	}

    @Override
    public Font getFont()
    {
        Font font = this.font;
        if( font == null )
        {
            UIDefaults uiDefaults = UIManager.getDefaults();
            String key = "Table.font";
            font = ( Font )uiDefaults.get( key );
        }
        return font;
    }

    @Override
    public void setFont( Font font )
    {
        super.setFont( font );
        this.font = font;
    }
}
