package siarchive.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.sql.SQLException;

import junit.framework.Assert;

import org.junit.Test;

import siarchive.components.Position;
import siarchive.components.Status;
import siarchive.i18n.Language;
import siarchive.persistence.Account;
import siarchive.persistence.DatabaseFactory;
import siarchive.persistence.Note;
import siarchive.persistence.Planet;
import siarchive.persistence.Player;
import siarchive.persistence.dao.AccountDao;
import siarchive.persistence.dao.PlanetDao;
import siarchive.persistence.dao.PlayerDao;

public class PlanetTest
{

    @Test
    public void test() throws SQLException
    {
        DatabaseFactory.open();
        Account account;
        AccountDao accountDao = DatabaseFactory.getDao( AccountDao.class );
        Player player;
        PlayerDao playerDao = DatabaseFactory.getDao( PlayerDao.class );
        Planet planet;
        PlanetDao planetDao = DatabaseFactory.getDao( PlanetDao.class );

        account = new Account();
        account.setName( "testAccount1" );
        account.setLanguage( Language.german );
        account.setIndex( 1 );
        account.setDefault( false );
        accountDao.save( account );

        player = new Player();
        player.setName( "Bluehawk" + 1 );
        player.setAccount( account.getId() );
        player.setPoints( 30000000L + 1 * 1000);
        player.setUpdateTime( System.currentTimeMillis() );
        player.setStatus( Status.enemy );
        player.setSecondaryStatus( null );
        playerDao.save( player );

        planet = new Planet();
        planet.setName( "Colony");
        planet.setAccount( account.getId() );
        planet.setOwner( player );
        planet.setPosition( Position.createId( 1, 1, 1 ) );
        planet.setUpdateTime( System.currentTimeMillis() );
        planetDao.save( planet );

        account = new Account();
        account.setName( "testAccount2" );
        account.setLanguage( Language.german );
        account.setIndex( 1 );
        account.setDefault( false );
        accountDao.save( account );

        player = new Player();
        player.setName( "Bluehawk" + 2 );
        player.setAccount( account.getId() );
        player.setPoints( 30000000L + 2 * 1000);
        player.setUpdateTime( System.currentTimeMillis() );
        player.setStatus( Status.enemy );
        player.setSecondaryStatus( null );
        playerDao.save( player );
        
        planet = new Planet();
        planet.setName( "Colony");
        planet.setAccount( account.getId() );
        planet.setOwner( player );
        planet.setPosition( Position.createId( 1, 1, 2 ) );
        planet.setUpdateTime( System.currentTimeMillis() );
        planetDao.save( planet );
        DatabaseFactory.commit();

        
        assertEquals( 2, planetDao.count() );
        assertEquals( 1, planetDao.countByAccount( account.getId() ) );
        planet = planetDao.find( account.getId(), Position.createId( 1, 1, 1 ) );
        assertEquals( null, planet );
        planet = planetDao.find( account.getId(), Position.createId( 1, 1, 2 ) );
        assertNotNull( planet );

        Note notes = new Note();
        notes.setText( "some Note" );
        planet.setNotes( notes );
        planetDao.save( planet );
        DatabaseFactory.commit();

        planet = new Planet();
        planet.setAccount( account.getId() );
        planet.setName( "Home");
        planet.setOwner( player );
        planet.setPosition( Position.createId( 1, 1, 2 ) );
        planet.setUpdateTime( System.currentTimeMillis() );
        planet.setRetainNotes( true );
        planetDao.save( planet );
        DatabaseFactory.commit();

        planet = planetDao.find( account.getId(), Position.createId( 1, 1, 2 ) );
        assertNotNull( planet );
        assertNotNull( planet.getNotes() );
        assertEquals( "some Note", planet.getNotes().getText() );
        
        account = accountDao.find( "testAccount1" );
        accountDao.delete( account );
        account = accountDao.find( "testAccount2" );
        accountDao.delete( account );
        DatabaseFactory.commit();
        assertEquals( 0, accountDao.count() );
        assertEquals( 0, planetDao.count() );
        
        DatabaseFactory.commit();
        DatabaseFactory.close();
    }

    @Test
    public void timeTest() throws SQLException
    {
        DatabaseFactory.open();
        Account account;
        AccountDao accountDao = DatabaseFactory.getDao( AccountDao.class );
        Player player;
        PlayerDao playerDao = DatabaseFactory.getDao( PlayerDao.class );
        Planet planet;
        PlanetDao planetDao = DatabaseFactory.getDao( PlanetDao.class );

        account = new Account();
        account.setName( "timeTestAccount" );
        account.setLanguage( Language.german );
        account.setIndex( 1 );
        account.setDefault( false );
        accountDao.save( account );

        player = new Player();
        player.setName( "TimeTest");
        player.setAccount( account.getId() );
        player.setPoints( 30000000L + 1 * 1000);
        player.setUpdateTime( System.currentTimeMillis() );
        player.setStatus( Status.enemy );
        player.setSecondaryStatus( null );
        playerDao.save( player );

        long time = 1000000000L;
        
        for(int sys = 1; sys < 11; sys++)
        {
	        for(int pla = 1; pla < 17; pla += 5 )
	        {
		        planet = new Planet();
		        planet.setName( "Colony");
		        planet.setAccount( account.getId() );
		        planet.setOwner( player );
		        planet.setPosition( Position.createId( 1, sys, pla ) );
		        planet.setUpdateTime( time );
		        planetDao.save( planet );
		        time += 1000000;
	        }
        }

        int start;
        int end;
        start = Position.createId(1, 1, 1);
        end = Position.createId(1, 1, 16);
        Long min1 = planetDao.minTimeByPosition(account.getId(), start, end);
        Assert.assertEquals(Long.valueOf(1000000000L), min1);

        start = Position.createId(1, 2, 1);
        end = Position.createId(1, 2, 16);
        Long min2 = planetDao.minTimeByPosition(account.getId(), start, end);
        Assert.assertEquals(Long.valueOf(1004000000L), min2);
        
        start = Position.createId(1, 8, 1);
        end = Position.createId(1, 8, 16);
        Long min8 = planetDao.minTimeByPosition(account.getId(), start, end);
        Assert.assertEquals(Long.valueOf(1028000000L), min8);

        start = Position.createId(1, 12, 1);
        end = Position.createId(1, 12, 16);
        Long min12 = planetDao.minTimeByPosition(account.getId(), start, end);
        Assert.assertEquals(null, min12);

        account = accountDao.find( "timeTestAccount" );
        accountDao.delete( account );
        DatabaseFactory.commit();
        DatabaseFactory.close();
    }

}
